'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _deepEqual = require('deep-equal');

var _deepEqual2 = _interopRequireDefault(_deepEqual);

require('array.from');

var _save = require('./save');

var _save2 = _interopRequireDefault(_save);

var SET_REDUCER = 'redux-pouchdb/SET_REDUCER';
exports.SET_REDUCER = SET_REDUCER;
var INIT = '@@redux-pouchdb/INIT';

exports.INIT = INIT;
var saveReducer = undefined;
var isInitialized = false;
var persistentStore = function persistentStore(db) {
  return function (storeCreator) {
    return function (reducer, initialState) {

      var store = storeCreator(reducer, initialState);

      saveReducer = (0, _save2['default'])(db);

      var setReducer = function setReducer(doc) {
        var _id = doc._id;
        var _rev = doc._rev;
        var state = doc.state;

        store.dispatch({
          type: SET_REDUCER,
          reducer: _id,
          state: state,
          _rev: _rev
        });
      };

      db.allDocs({ include_docs: true }).then(function (res) {
        isInitialized = true;

        var promises = res.rows.map(function (row) {
          return setReducer(row.doc);
        });
        return Promise.all(promises);
      }).then(function () {
        store.dispatch({
          type: INIT
        });

        return db.changes({
          include_docs: true,
          live: true,
          since: 'now'
        }).on('change', function (change) {
          var storeState = store.getState();

          if (change.doc.state) {
            if (!(0, _deepEqual2['default'])(change.doc.state, storeState)) {
              setReducer(change.doc);
            }
          } else {
            saveReducer(change.doc._id, store.getState());
          }
        });
      })['catch'](console.error.bind(console));

      return store;
    };
  };
};

exports.persistentStore = persistentStore;
var persistentReducer = function persistentReducer(reducer) {
  var lastState = undefined;

  return function (state, action) {
    if (action.type === SET_REDUCER && action.reducer === reducer.name && action.state) {

      lastState = action.state;
      return reducer(action.state, action);
    }

    var reducedState = reducer(state, action);

    if (isInitialized && !(0, _deepEqual2['default'])(reducedState, lastState)) {
      lastState = reducedState;

      saveReducer(reducer.name, reducedState);
    }

    return reducedState;
  };
};
exports.persistentReducer = persistentReducer;